/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.lib;

import java.io.UnsupportedEncodingException;

import static dk.vajhoej.vms.call.VMS.*;
import static dk.vajhoej.vms.call.lib.VMSStatus.*;

import dk.vajhoej.record.RecordException;
import dk.vajhoej.vms.call.VMS.CharacterString;
import dk.vajhoej.vms.call.VMS.LongWord;
import dk.vajhoej.vms.call.VMSLibrary;
import dk.vajhoej.vms.call.RFA;

/**
 * Class LBRRTL is a wrapper library for LBR$ functions.
 * 
 * See VMS documentation for semantics.
 */
public class LBRRTL extends VMSLibrary {
	/**
	 * Class LBR_IDX encapsulate LBR$ index and wrap a series of LBR$ calls.
	 *
	 */
	public static class LBR_IDX {
		private LongWord idx;
		private Block rfa;
		/**
		 * Call LBR$INI_CONTROL.
		 * @param access LBR$C_CREATE / LBR$C_READ / LBR$C_UPDATE
		 * @param typ LBR$C_TYP_OBJ / LBR$C_TYP_MLB / LBR$C_TYP_HLP / LBR$C_TYP_TXT
		 * @return return status code
		 */
		public Result<Void> LBR$INI_CONTROL(int access, int typ) {
			idx = new LongWord(0);
	        int sts = call("LBRSHR", "LBR$INI_CONTROL", pass(idx).byReference().writeOnly(),
	        		                                    pass(new LongWord(access)).byReference().readOnly(),
	        		                                    pass(new LongWord(typ)).byReference().readOnly());
	        return new Result<Void>(sts, null);
		}
		/**
		 * Call LBR$OPEN.
		 * @param fnm file name
		 * @return return status code
		 * @throws UnsupportedEncodingException if file name can not be decoded
		 */
		public Result<Void> LBR$OPEN(String fnm) throws UnsupportedEncodingException {
	        int sts = call("LBRSHR", "LBR$OPEN", pass(idx).byReference().readOnly(),
	                                             pass(new CharacterString(fnm)).byDescriptor().readOnly());
	        return new Result<Void>(sts, null);
		}
		/**
		 * Call LBR$LOOKUP_KEY.
		 * @param key key to lookup
		 * @return return status code
		 * @throws RecordException if internal problem
		 * @throws UnsupportedEncodingException if key can not be decoded
		 */
		public Result<Void> LBR$LOOKUP_KEY(String key) throws RecordException, UnsupportedEncodingException {
			rfa = new Block(new RFA());
			int sts = call("LBRSHR", "LBR$LOOKUP_KEY", pass(idx).byReference().readOnly(),
					                                   pass(new CharacterString(key)).byDescriptor().readOnly(),
					                                   pass(rfa).byReference().writeOnly());
			return new Result<Void>(sts, null);
		}
		/**
		 * Call LBR$GET_RECORD.
		 * @return return status code + record
		 * @throws UnsupportedEncodingException if record can not be encoded
		 */
		public Result<String> LBR$GET_RECORD() throws UnsupportedEncodingException {
			CharacterString buf = new CharacterString(32767);
	        int sts = call("LBRSHR", "LBR$GET_RECORD", pass(idx).byReference().readOnly(),
	        		                                   pass(buf).byDescriptor().writeOnly());
	        String res = createTrimmedString(buf);
	        return new Result<String>(sts, res);
		}
		/**
		 * Call LBR$GET_HELP.
		 * @param topics help topics
		 * @return return status code + help text
		 * @throws UnsupportedEncodingException if help topics can not be decoded or help text can not be encoded
		 */
		public Result<String> LBR$GET_HELP(String... topics) throws UnsupportedEncodingException {
			Routine r = new Routine(ROUTINE_4ARGS_1STRETDATA);
			int sts;
			switch(topics.length) {
				case 1:
					sts = call("LBRSHR", "LBR$GET_HELP", pass(idx).byReference().readOnly(),
	                        							 MISSING, 
	                        							 pass(r).byReference().readOnly(),
	                        							 MISSING,
	                        							 pass(new CharacterString(topics[0])).byDescriptor().readOnly());
	                break;
				case 2:
					sts = call("LBRSHR", "LBR$GET_HELP", pass(idx).byReference().readOnly(),
	                        							 MISSING, 
	                        							 pass(r).byReference().readOnly(),
	                        							 MISSING,
	                        							 pass(new CharacterString(topics[0])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[1])).byDescriptor().readOnly());
	                break;
				case 3:
					sts = call("LBRSHR", "LBR$GET_HELP", pass(idx).byReference().readOnly(),
	                        							 MISSING, 
	                        							 pass(r).byReference().readOnly(),
	                        							 MISSING,
	                        							 pass(new CharacterString(topics[0])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[1])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[2])).byDescriptor().readOnly());
	                break;
				case 4:
					sts = call("LBRSHR", "LBR$GET_HELP", pass(idx).byReference().readOnly(),
	                        							 MISSING, 
	                        							 pass(r).byReference().readOnly(),
	                        							 MISSING,
	                        							 pass(new CharacterString(topics[0])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[1])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[2])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[3])).byDescriptor().readOnly());
	                break;
				case 5:
					sts = call("LBRSHR", "LBR$GET_HELP", pass(idx).byReference().readOnly(),
	                        							 MISSING, 
	                        							 pass(r).byReference().readOnly(),
	                        							 MISSING,
	                        							 pass(new CharacterString(topics[0])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[1])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[2])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[3])).byDescriptor().readOnly(),
	                        							 pass(new CharacterString(topics[4])).byDescriptor().readOnly());
	                break;
	            default:
	            	return new Result<String>(SS$_INSFARG, "");
			}
			return new Result<String>(sts, createTrimmedString(r.getData()));
					                                 
		}
		/**
		 * Call LBR$CLOSE.
		 * @return return status code
		 */
		public Result<Void> LBR$CLOSE() {
	        int sts = call("LBRSHR", "LBR$CLOSE", pass(idx).byReference().readOnly());
	        return new Result<Void>(sts, null);
		}
	}
}
