/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.lib;

import static dk.vajhoej.vms.call.VMS.*;
import static dk.vajhoej.vms.call.lib.VMSCodes.*;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import dk.vajhoej.record.RecordException;
import dk.vajhoej.vms.call.IOSB;
import dk.vajhoej.vms.call.VMSLibrary;
import dk.vajhoej.vms.call.VMS.CharacterString;
import dk.vajhoej.vms.call.VMS.Item32;
import dk.vajhoej.vms.call.VMS.Item32CharacterStringIn;
import dk.vajhoej.vms.call.VMS.ItemList32;
import dk.vajhoej.vms.call.VMS.LongWord;
import dk.vajhoej.vms.call.VMS.Word;

/**
 * Class SYS is a wrapper library for system services (SYS$ functions).
 * 
 * See VMS documentation for semantics.
 */
public class SYS extends VMSLibrary {
	/**
	 * Call SYS$GETDVIW to get a list of items.
	 * @param devnam device name
	 * @param itms array of items
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<Integer,Object>> SYS$GETDVIW(String devnam, int[] itms) throws UnsupportedEncodingException, RecordException {
		Item32[] itmarr = new Item32[itms.length];
		for(int i = 0; i < itms.length; i++) {
			switch(itms[i]) {
	            case DVI$_DEVTYPE:
	            case DVI$_ERRCNT:
	            case DVI$_MAXBLOCK:
	            case DVI$_FREEBLOCKS:
			    	itmarr[i] = new Item32LongWordOut(itms[i]);
			    	break;
	            case DVI$_VOLNAM:
			    	itmarr[i] = new Item32CharacterStringOut(itms[i], 1024);
			    	break;
			    default:
			    	throw new RuntimeException("Unsupported GETJPI code: " + itms[i]);
			}
		}
		ItemList32 itmlst = new ItemList32(itmarr);
		Block iosb = new Block(new IOSB());
		int sts = call("SYS$PUBLIC_VECTORS", "SYS$GETDVIW", MISSING,
				                                            MISSING,
				                                            pass(new CharacterString(devnam)).byDescriptor().readOnly(),
				                                            pass(itmlst).byReference().readOnly(),
				                                            pass(iosb).byReference().writeOnly(),
				                                            MISSING,
				                                            MISSING,
				                                            MISSING,
				                                            MISSING);
		Map<Integer,Object> res = new HashMap<Integer,Object>();
		for(int i = 0; i < itms.length; i++) {
			switch(itms[i]) {
	            case DVI$_DEVTYPE:
	            case DVI$_ERRCNT:
	            case DVI$_MAXBLOCK:
	            case DVI$_FREEBLOCKS:
			    	LongWord lw = (LongWord)itmarr[i].getBufadr();
			    	res.put(itms[i], lw.getValue());
			    	break;
	            case DVI$_VOLNAM:
			    	CharacterString cs = (CharacterString)itmarr[i].getBufadr();
			    	res.put(itms[i], createTrimmedString(cs));
			    	break;
			    default:
			    	throw new RuntimeException("Unsupported GETDVI code: " + itms[i]);
			}
		}
		return new Result<Map<Integer,Object>>(combineStatus(sts, iosb), res);
	}
	private static Result<Map<Integer,Object>> SYS$GETJPIW(LongWord pid, int[] itms) throws UnsupportedEncodingException, RecordException {
		Item32[] itmarr = new Item32[itms.length];
		for(int i = 0; i < itms.length; i++) {
			switch(itms[i]) {
			    case JPI$_PRI:
			    case JPI$_UIC:
			    case JPI$_STATE:
			    case JPI$_PID:
			    case JPI$_DIRIO:
			    case JPI$_BUFIO:
			    case JPI$_WSSIZE:
			    	itmarr[i] = new Item32LongWordOut(itms[i]);
			    	break;
			    case JPI$_CURPRIV:
			    case JPI$_CPUTIM:
			    case JPI$_LOGINTIM:
			    	itmarr[i] = new Item32QuadWordOut(itms[i]);
			    	break;
			    case JPI$_PRCNAM:
			    case JPI$_TERMINAL:
			    case JPI$_NODENAME:
			    case JPI$_USERNAME:
			    case JPI$_IMAGNAME:
			    	itmarr[i] = new Item32CharacterStringOut(itms[i], 1024);
			    	break;
			    default:
			    	throw new RuntimeException("Unsupported GETJPI code: " + itms[i]);
			}
		}
		ItemList32 itmlst = new ItemList32(itmarr);
		Block iosb = new Block(new IOSB());
		int sts = call("SYS$PUBLIC_VECTORS", "SYS$GETJPIW", MISSING,
				                                            pass(pid).byReference().readWrite(),
				                                            MISSING,
				                                            pass(itmlst).byReference().readOnly(),
				                                            pass(iosb).byReference().writeOnly(),
				                                            MISSING,
				                                            MISSING);
		Map<Integer,Object> res = new HashMap<Integer,Object>();
		for(int i = 0; i < itms.length; i++) {
			switch(itms[i]) {
			    case JPI$_PRI:
			    case JPI$_UIC:
			    case JPI$_STATE:
			    case JPI$_PID:
			    case JPI$_DIRIO:
			    case JPI$_BUFIO:
			    case JPI$_WSSIZE:
			    	LongWord lw = (LongWord)itmarr[i].getBufadr();
			    	res.put(itms[i], lw.getValue());
			    	break;
			    case JPI$_CURPRIV:
			    case JPI$_CPUTIM:
			    case JPI$_LOGINTIM:
			    	QuadWord qw = (QuadWord)itmarr[i].getBufadr();
			    	res.put(itms[i], qw.getValue());
			    	break;
			    case JPI$_PRCNAM:
			    case JPI$_TERMINAL:
			    case JPI$_NODENAME:
			    case JPI$_USERNAME:
			    case JPI$_IMAGNAME:
			    	CharacterString cs = (CharacterString)itmarr[i].getBufadr();
			    	res.put(itms[i], createTrimmedString(cs));
			    	break;
			    default:
			    	throw new RuntimeException("Unsupported GETJPI code: " + itms[i]);
			}
		}
		return new Result<Map<Integer,Object>>(combineStatus(sts, iosb), res);
	}
	/**
	 * Call SYS$GETJPIW to get a list of items.
	 * @param pid PID
	 * @param itms array of items
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<Integer,Object>> SYS$GETJPIW(int pid, int[] itms) throws UnsupportedEncodingException, RecordException {
		return SYS$GETJPIW(new LongWord(pid), itms);
	}
	/**
	 * Call SYS$IDTOASC to get name for identifier.
	 * @param id identifier
	 * @return return status code + identifier name
	 * @throws UnsupportedEncodingException if error encoding identifier name
	 */
	public static Result<String> SYS$IDTOASC(int id) throws UnsupportedEncodingException {
        CharacterString buf = new CharacterString(255);
        Word retlen = new Word();
        int sts = call("SYS$PUBLIC_VECTORS", "SYS$IDTOASC", pass(new LongWord(id)).byValue().readOnly(),
                                                            pass(retlen).byReference().writeOnly(),
                                                            pass(buf).byDescriptor().writeOnly(),
                                                            MISSING,
                                                            MISSING,
                                                            MISSING);
        String res = createString(buf, retlen);
        return new Result<String>(sts, res);
	}
	/**
	 * Class ID_CTX represents the context for SYS$IDTOASC calls.
	 */
	public static class ID_CTX {
		private LongWord ctx = new LongWord();
		/**
		 * Call SYS$IDTOASC to return next identifier.
		 * @return return status code + identifier
		 * @throws UnsupportedEncodingException if error encoding identifier name
		 */
		public Result<Pair<Integer, String>> SYS$IDTOASC() throws UnsupportedEncodingException {
	        CharacterString buf = new CharacterString(255);
	        Word retlen = new Word();
	        LongWord resid = new LongWord();
	        int sts = call("SYS$PUBLIC_VECTORS", "SYS$IDTOASC", pass(new LongWord(-1)).byValue().readOnly(),
	                                                            pass(retlen).byReference().writeOnly(),
	                                                            pass(buf).byDescriptor().writeOnly(),
	                                                            pass(resid).byReference().writeOnly(),
	                                                            MISSING,
	                                                            pass(ctx).byReference().readWrite());
	        String res = createString(buf, retlen);
	        return new Result<Pair<Integer,String>>(sts, new Pair<Integer,String>(resid.getValue(), res));
		}
	}
	/**
	 * Call SYS$GETUAI to get a list of items.
	 * @param usrnam username
	 * @param itms array of items
	 * @return return status code +  Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<Integer,Object>> SYS$GETUAI(String usrnam, int[] itms) throws UnsupportedEncodingException, RecordException {
		Item32[] itmarr = new Item32[itms.length];
		for(int i = 0; i < itms.length; i++) {
			switch(itms[i]) {
				case UAI$_UIC:
				case UAI$_LOGFAILS:
				case UAI$_SALT:
				case UAI$_ENCRYPT:
				case UAI$_PRI:
				case UAI$_MAXJOBS:
				case UAI$_PRCCNT:
				case UAI$_WSQUOTA:
				case UAI$_WSEXTENT:
				case UAI$_PGFLQUOTA:
			    	itmarr[i] = new Item32LongWordOut(itms[i]);
			    	break;
				case UAI$_PWD:
				case UAI$_PWD_DATE:
				case UAI$_PRIV:
				case UAI$_DEF_PRIV:
			    	itmarr[i] = new Item32QuadWordOut(itms[i]);
			    	break;
				case UAI$_USERNAME:
				case UAI$_ACCOUNT:
				case UAI$_OWNER:
				case UAI$_DEFDEV:
				case UAI$_DEFDIR:
			    	itmarr[i] = new Item32CharacterStringOut(itms[i], 1024);
			    	break;
			    default:
			    	throw new RuntimeException("Unsupported GETJPI code: " + itms[i]);
			}
		}
		ItemList32 itmlst = new ItemList32(itmarr);
		int sts = call("SECURESHRP", "SYS$GETUAI", MISSING,
				                                   MISSING,
				                                   pass(new CharacterString(usrnam)).byDescriptor().readOnly(),
			                                       pass(itmlst).byReference().readOnly(),
			                                       MISSING,
			                                       MISSING,
				                                   MISSING);
		Map<Integer,Object> res = new HashMap<Integer,Object>();
		for(int i = 0; i < itms.length; i++) {
			switch(itms[i]) {
				case UAI$_UIC:
				case UAI$_LOGFAILS:
				case UAI$_SALT:
				case UAI$_ENCRYPT:
				case UAI$_PRI:
				case UAI$_MAXJOBS:
				case UAI$_PRCCNT:
				case UAI$_WSQUOTA:
				case UAI$_WSEXTENT:
				case UAI$_PGFLQUOTA:
			    	LongWord lw = (LongWord)itmarr[i].getBufadr();
			    	res.put(itms[i], lw.getValue());
			    	break;
				case UAI$_PWD:
				case UAI$_PWD_DATE:
				case UAI$_PRIV:
				case UAI$_DEF_PRIV:
			    	QuadWord qw = (QuadWord)itmarr[i].getBufadr();
			    	res.put(itms[i], qw.getValue());
			    	break;
				case UAI$_USERNAME:
				case UAI$_ACCOUNT:
				case UAI$_OWNER:
				case UAI$_DEFDEV:
				case UAI$_DEFDIR:
			    	CharacterString cs = (CharacterString)itmarr[i].getBufadr();
			    	res.put(itms[i], createTrimmedString(cs));
			    	break;
			    default:
			    	throw new RuntimeException("Unsupported GETUAI code: " + itms[i]);
			}
		}
		return new Result<Map<Integer,Object>>(sts, res);
	}
	/**
	 * Class PID_CTX represents the context for SYS$PROCESS_SCAN calls.
	 */
	public static class PID_CTX {
		private LongWord ctx = new LongWord();
		/**
		 * Call SYS$PROCESS_SCAN to find next matching PID.
		 * @param code code for SYS$PROCESS_SCAN
		 * @param value value for SYS$PROCESS_SCAN
		 * @return return status code
		 * @throws UnsupportedEncodingException if value can not be decoded
		 */
		public Result<Void> SYS$PROCESS_SCAN(int code, String value) throws UnsupportedEncodingException {
			Result<Integer> res = SYS.SYS$PROCESS_SCAN(code, value);
			this.ctx = new LongWord(res.getResult());
			return new Result<Void>(res.getStatus(), null);
		}
		/**
		 * Call SYS$GETJPIW to get lost of items.
		 * @param itms array of items
		 * @return return status code + Java Map of items
		 * @throws UnsupportedEncodingException if some string values can not be encoded
		 * @throws RecordException if internal error
		 */
		public Result<Map<Integer,Object>> SYS$GETJPIW(int[] itms) throws UnsupportedEncodingException, RecordException {
			return SYS.SYS$GETJPIW(ctx, itms);
		}
	}
	/**
	 * Call SYS$PROCESS_SCAN to get first matching PID.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @return return status code + PID
	 * @throws UnsupportedEncodingException if value can not be decoded
	 */
	public static Result<Integer> SYS$PROCESS_SCAN(int code, String value) throws UnsupportedEncodingException {
		LongWord cntx = new LongWord();
		ItemList32 itmlst = new ItemList32(new Item32[] { new Item32CharacterStringIn(code, new CharacterString(value), value.contains("*") ? PSCAN$M_WILDCARD : 0) });
		int sts = call("SYS$PUBLIC_VECTORS", "SYS$PROCESS_SCAN", pass(cntx).byReference().readWrite(),
				                                                 pass(itmlst).byReference().readOnly());
		return new Result<Integer>(sts, cntx.getValue());
	}
}
