/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.script;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import dk.vajhoej.record.RecordException;
import dk.vajhoej.vms.call.lib.SYS_Util;

/**
 * Class SYS_Script is a script friendly wrapper library for system services (SYS$ functions).
 */
public class SYS_Script extends SYS_Util {
	/**
	 * Call SYS$GETDVIW to get a list of items.
	 * @param devnam device name
	 * @param itms array of items
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS$GETDVIW(String devnam, String[] itms) throws UnsupportedEncodingException, RecordException {
		int[] numitms = VMSCodesNames.nameToNumber("DVI", itms);
		Result<Map<Integer,Object>> res = SYS$GETDVIW(devnam, numitms);
		return new Result<Map<String,Object>>(res.getStatus(), VMSCodesNames.numberToName("DVI", res.getResult()));
	}
	/**
	 * Call SYS$GETDVIW to get a list of all items.
	 * @param devnam device name
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS$GETDVIW(String devnam) throws UnsupportedEncodingException, RecordException {
		int[] numitms = VMSCodesNames.all("DVI");
		Result<Map<Integer,Object>> res = SYS$GETDVIW(devnam, numitms);
		return new Result<Map<String,Object>>(res.getStatus(), VMSCodesNames.numberToName("DVI", res.getResult()));
	}
	/**
	 * Call SYS$GETJPIW to get a list of items.
	 * @param pid PID
	 * @param itms array of items
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS$GETJPIW(int pid, String[] itms) throws UnsupportedEncodingException, RecordException {
		int[] numitms = VMSCodesNames.nameToNumber("JPI", itms);
		Result<Map<Integer,Object>> res = SYS$GETJPIW(pid, numitms);
		return new Result<Map<String,Object>>(res.getStatus(), VMSCodesNames.numberToName("JPI", res.getResult()));
	}
	/**
	 * Call SYS$GETJPIW to get a list of all items.
	 * @param pid PID
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS$GETJPIW(int pid) throws UnsupportedEncodingException, RecordException {
		int[] numitms = VMSCodesNames.all("JPI");
		Result<Map<Integer,Object>> res = SYS$GETJPIW(pid, numitms);
		return new Result<Map<String,Object>>(res.getStatus(), VMSCodesNames.numberToName("JPI", res.getResult()));
	}
	/**
	 * Call SYS$GETUAI to get a list of items.
	 * @param usrnam username
	 * @param itms array of items
	 * @return return status code +  Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS$GETUAI(String usrnam, String[] itms) throws UnsupportedEncodingException, RecordException {
		int[] numitms = VMSCodesNames.nameToNumber("UAI", itms);
		Result<Map<Integer,Object>> res = SYS$GETUAI(usrnam, numitms);
		return new Result<Map<String,Object>>(res.getStatus(), VMSCodesNames.numberToName("UAI", res.getResult()));
	}
	/**
	 * Call SYS$GETUAI to get a list of all items.
	 * @param usrnam username
	 * @return return status code +  Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS$GETUAI(String usrnam) throws UnsupportedEncodingException, RecordException {
		int[] numitms = VMSCodesNames.all("UAI");
		Result<Map<Integer,Object>> res = SYS$GETUAI(usrnam, numitms);
		return new Result<Map<String,Object>>(res.getStatus(), VMSCodesNames.numberToName("UAI", res.getResult()));
	}
	/**
	 * Call SYS$IDTOASC to get list of all identifiers.
	 * @param inclneg include negative (identifiers that are not UIC's)
	 * @return list of all identifiers
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 */
	public static List<String> SYS$IDTOASC_AsList(boolean inclneg) throws UnsupportedEncodingException {
		return new ArrayList<String>(SYS$IDTOASC(inclneg).values());
	}
	/**
	 * Call SYS$PROCESS_SCAN to get all PID's and call SYS$GETJPIW for each PID to get list of items.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @param itms array of items
	 * @return Java List of Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static List<Map<String,Object>> SYS$PROCESS_SCAN_and_SYS$GETJPIW_AsList(String code, String value, String[] itms) throws UnsupportedEncodingException, RecordException {
		int[] numitms = VMSCodesNames.nameToNumber("JPI", itms);
		List<Map<Integer,Object>> res = SYS$PROCESS_SCAN_and_SYS$GETJPIW(VMSCodesNames.nameToNumber("PSCAN", code), value, numitms);
		return VMSCodesNames.numberToName("JPI", res);
	}
	/**
	 * Call SYS$PROCESS_SCAN to get all PID's and call SYS$GETJPIW for each PID to get list of all items.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @return Java List of Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static List<Map<String,Object>> SYS$PROCESS_SCAN_and_SYS$GETJPIW_AsList(String code, String value) throws UnsupportedEncodingException, RecordException {
		int[] numitms = VMSCodesNames.all("JPI");
		List<Map<Integer,Object>> res = SYS$PROCESS_SCAN_and_SYS$GETJPIW(VMSCodesNames.nameToNumber("PSCAN", code), value, numitms);
		return VMSCodesNames.numberToName("JPI", res);
	}
}
