/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.script.js;

import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;

import org.mozilla.javascript.NativeArray;
import org.mozilla.javascript.NativeObject;

import dk.vajhoej.record.RecordException;
import dk.vajhoej.vms.call.script.SYS_Script;

/**
 * Class SYS_JS is a JavaScript friendly wrapper library for system services (SYS$ functions).
 */
public class SYS_JS extends SYS_Script{
	/**
	 * Call SYS$GETDVIW to get a list of items.
	 * @param devnam device name
	 * @param itms array of items
	 * @return return status code + JavaScript object with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<NativeObject> SYS$GETDVIW(String devnam, NativeArray itms) throws UnsupportedEncodingException, RecordException {
		Result<Map<String,Object>>  res = SYS$GETDVIW(devnam, Util.convertStringArray(itms));
		return new Result<NativeObject>(res.getStatus(), Util.convertMap(res.getResult()));
	}
	/**
	 * Call SYS$GETJPIW to get a list of items.
	 * @param pid PID
	 * @param itms array of items
	 * @return return status code +  JavaScript object with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<NativeObject> SYS$GETJPIW(int pid, NativeArray itms) throws UnsupportedEncodingException, RecordException {
		Result<Map<String,Object>>  res = SYS$GETJPIW(pid, Util.convertStringArray(itms));
		return new Result<NativeObject>(res.getStatus(), Util.convertMap(res.getResult()));
	}
	/**
	 * Call SYS$GETUAI to get a list of items.
	 * @param usrnam username
	 * @param itms array of items
	 * @return return status code +  JavaScript object with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<NativeObject> SYS$GETUAI(String usrnam, NativeArray itms) throws UnsupportedEncodingException, RecordException {
		Result<Map<String,Object>>  res = SYS$GETUAI(usrnam, Util.convertStringArray(itms));
		return new Result<NativeObject>(res.getStatus(), Util.convertMap(res.getResult()));
	}
	/**
	 * Call SYS$IDTOASC to get list of all identifiers.
	 * @param inclneg include negative (identifiers that are not UIC's)
	 * @return array of all identifiers
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 */
	public static NativeArray SYS$IDTOASC_AsArray(boolean inclneg) throws UnsupportedEncodingException {
		return Util.convertList(SYS$IDTOASC_AsList(inclneg));
	}
	/**
	 * Call SYS$PROCESS_SCAN to get all PID's and call SYS$GETJPIW for each PID to get list of items.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @param itms array of itms
	 * @return JavaScript array of JavaScript objects with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static NativeArray SYS$PROCESS_SCAN_and_SYS$GETJPIW_AsArray(String code, String value, NativeArray itms) throws UnsupportedEncodingException, RecordException {
		List<Map<String,Object>> res = SYS$PROCESS_SCAN_and_SYS$GETJPIW_AsList(code, value, Util.convertStringArray(itms));
		return Util.convertListMap(res);
	}
}
