/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.script.py;

import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;

import dk.vajhoej.record.RecordException;
import dk.vajhoej.vms.call.script.SYS_Script;

/**
 * Class SYS_Py is a Python friendly wrapper library for system services (SYS$ functions).
 */
public class SYS_Py extends SYS_Script {
	/**
	 * Call SYS$GETDVIW to get a list of items.
	 * @param devnam device name
	 * @param itms array of items
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETDVIW(String devnam, String[] itms) throws UnsupportedEncodingException, RecordException {
		return SYS$GETDVIW(devnam, itms);
	}
	/**
	 * Call SYS$GETJPIW to get a list of items.
	 * @param pid PID
	 * @param itms array of items
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETJPIW(int pid, String[] itms) throws UnsupportedEncodingException, RecordException {
		return SYS$GETJPIW(pid, itms);
	}
	/**
	 * Call SYS$IDTOASC to get name for identifier.
	 * @param id identifier
	 * @return return status code + identifier name
	 * @throws UnsupportedEncodingException if error encoding identifier name
	 */
	public static Result<String> SYS__IDTOASC(int id) throws UnsupportedEncodingException {
		return SYS$IDTOASC(id);
	}
	/**
	 * Class ID_CTX represents the context for SYS$IDTOASC calls.
	 */
	public static class ID_CTX extends dk.vajhoej.vms.call.lib.SYS.ID_CTX {
		/**
		 * Call SYS$IDTOASC to return next identifier.
		 * @return return status code + identifier
		 * @throws UnsupportedEncodingException if error encoding identifier name
		 */
		public Result<Pair<Integer, String>> SYS__IDTOASC() throws UnsupportedEncodingException {
			return SYS$IDTOASC();
		}
	}
	/**
	 * Call SYS$GETUAI to get a list of items.
	 * @param usrnam username
	 * @param itms array of items
	 * @return return status code +  Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETUAI(String usrnam, String[] itms) throws UnsupportedEncodingException, RecordException {
		return SYS$GETUAI(usrnam, itms);
	}
	/**
	 * Call SYS$PROCESS_SCAN to get first matching PID.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @return return status code + PID
	 * @throws UnsupportedEncodingException if value can not be decoded
	 */
	public static Result<Integer> SYS__PROCESS_SCAN(int code, String value) throws UnsupportedEncodingException {
		return SYS$PROCESS_SCAN(code, value);
	}
	/**
	 * Call SYS$IDTOASC to get map of identifiers to identifier names.
	 * @param inclneg include negative (identifiers that are not UIC's)
	 * @return map of identifiers to identifier names
	 * @throws UnsupportedEncodingException if error encoding UIC name
	 */
	public static Map<Integer,String> SYS__IDTOASC(boolean inclneg) throws UnsupportedEncodingException {
		return SYS$IDTOASC(inclneg);
	}
	/**
	 * Call SYS$GETDVIW to get a list of all items.
	 * @param devnam device name
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETDVIW(String devnam) throws UnsupportedEncodingException, RecordException {
		return SYS$GETDVIW(devnam);
	}
	/**
	 * Call SYS$GETJPIW to get a list of all items.
	 * @param pid PID
	 * @return return status code + Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETJPIW(int pid) throws UnsupportedEncodingException, RecordException {
		return SYS$GETJPIW(pid);
	}
	/**
	 * Call SYS$GETUAI to get a list of all items.
	 * @param usrnam username
	 * @return return status code +  Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static Result<Map<String,Object>> SYS__GETUAI(String usrnam) throws UnsupportedEncodingException, RecordException {
		return SYS$GETUAI(usrnam);
	}
	/**
	 * Call SYS$IDTOASC to get list of all identifiers.
	 * @param inclneg include negative (identifiers that are not UIC's)
	 * @return list of all identifiers
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 */
	public static List<String> SYS__IDTOASC_AsList(boolean inclneg) throws UnsupportedEncodingException {
		return SYS$IDTOASC_AsList(inclneg);
	}
	/**
	 * Call SYS$PROCESS_SCAN to get all PID's and call SYS$GETJPIW for each PID to get list of items.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @param itms array of items
	 * @return Java List of Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static List<Map<String,Object>> SYS__PROCESS_SCAN_and_SYS__GETJPIW_AsList(String code, String value, String[] itms) throws UnsupportedEncodingException, RecordException {
		return SYS$PROCESS_SCAN_and_SYS$GETJPIW_AsList(code, value, itms);
	}
	/**
	 * Call SYS$PROCESS_SCAN to get all PID's and call SYS$GETJPIW for each PID to get list of all items.
	 * @param code code for SYS$PROCESS_SCAN
	 * @param value value for SYS$PROCESS_SCAN
	 * @return Java List of Java Map with items
	 * @throws UnsupportedEncodingException if some string values can not be encoded
	 * @throws RecordException if internal error
	 */
	public static List<Map<String,Object>> SYS__PROCESS_SCAN_and_SYS__GETJPIW_AsList(String code, String value) throws UnsupportedEncodingException, RecordException {
		return SYS$PROCESS_SCAN_and_SYS$GETJPIW_AsList(code, value);
	}
}
